# -*- perl -*-
#
# Copyright (C) 1996 Ken MacLeod
# See the file COPYING for distribution terms.
#

use strict;

use Text::GDO::Transform;
use Text::GDO::Specification;

package Text::GDO::SpecTransform; # to appease Perl's `use' statement
package Text::GDO::Spec::Transform;
@Text::GDO::Spec::Transform::ISA = qw{Text::GDO::Transform};

sub new {
    my ($type, %args) = @_;

    my ($self) = \%args;

    bless $self, 'Text::GDO::Spec::Transform';

    $self->init;

    return $self;
}

sub init {
    my ($self) = @_;

    $self->{'map'} = {
	'SPEC'=>'SPEC',
	'HEAD'=>'HEAD',
	'RULES'=>'RULES',
	'RULE'=>'RULE',
	'STACK'=>'STACK',
	'SAVE'=>'SAVE',
	'PUSH'=>'PUSH',
	'QUERY'=>'QUERY',
	'ISA'=>'ISA',
	'STARTACTION'=>'STARTACTION',
	'ENDACTION'=>'ENDACTION',
	'BEFORE'=>'BEFORE',
	'PREFIX'=>'PREFIX',
	'SUFFIX'=>'SUFFIX',
	'AFTER'=>'AFTER',
	'CDATAFILTER'=>'CDATAFILTER',
	'SDATAFILTER'=>'SDATAFILTER',
	'ATTR'=>'ATTR',
	'STUFF'=>'STUFF'
    };
}

sub mapSPEC {
    my ($self, $data, $event, $element) = @_;
    my ($parent) = $data->{'diversion'}[-1];

    my ($gen_object) = new Text::GDO::Spec::Instance;
    $gen_object->{'parent'} = $parent;
    $gen_object->{'source'} = $event;
    push (@{$data->{'diversion'}}, $gen_object);
    $self->_map_instance ($data, $event);
    push (@{$parent->{'content'}}, pop (@{$data->{'diversion'}}));
}

sub mapHEAD {
    my ($self, $data, $event, $element) = @_;
    my ($parent) = $data->{'diversion'}[-1];

    $self->_map_instance ($data, $event);
}

sub mapRULES {
    my ($self, $data, $event, $element) = @_;
    my ($parent) = $data->{'diversion'}[-1];

    $self->_map_instance ($data, $event);
}

sub mapRULE {
    my ($self, $data, $event, $element) = @_;
    my ($parent) = $data->{'diversion'}[-1];

    my ($gen_object) = new Text::GDO::Spec::Rule;
    $gen_object->{'parent'} = $parent;
    $gen_object->{'source'} = $event;
    push (@{$data->{'diversion'}}, $gen_object);
    $self->_map_instance ($data, $event);
    push (@{$parent->{rules}}, pop (@{$data->{'diversion'}}));
}

sub mapQUERY {
    my ($self, $data, $event, $element) = @_;
    my ($parent) = $data->{'diversion'}[-1];

    my ($gen_object) = new Text::GDO::Spec::Thing;
    $gen_object->{'parent'} = $parent;
    $gen_object->{'source'} = $event;
    push (@{$data->{'diversion'}}, $gen_object);
    $self->_map_instance ($data, $event);
    $parent->{'query'} = pop (@{$data->{'diversion'}});
}

sub mapISA {
    my ($self, $data, $event, $element) = @_;
    my ($parent) = $data->{'diversion'}[-1];

    my ($gen_object) = new Text::GDO::Spec::Thing;
    $gen_object->{'parent'} = $parent;
    $gen_object->{'source'} = $event;
    push (@{$data->{'diversion'}}, $gen_object);
    $self->_map_instance ($data, $event);
    $parent->{'isa'} = pop (@{$data->{'diversion'}});
}

sub mapSAVE {
    my ($self, $data, $event, $element) = @_;
    my ($parent) = $data->{'diversion'}[-1];

    my ($gen_object) = new Text::GDO::Spec::Thing;
    $gen_object->{'parent'} = $parent;
    $gen_object->{'source'} = $event;
    push (@{$data->{'diversion'}}, $gen_object);
    $self->_map_instance ($data, $event);
    $parent->{'save'} = pop (@{$data->{'diversion'}});
}

sub mapPUSH {
    my ($self, $data, $event, $element) = @_;
    my ($parent) = $data->{'diversion'}[-1];

    my ($gen_object) = new Text::GDO::Spec::Thing;
    $gen_object->{'parent'} = $parent;
    $gen_object->{'source'} = $event;
    push (@{$data->{'diversion'}}, $gen_object);
    $self->_map_instance ($data, $event);
    $parent->{'push'} = pop (@{$data->{'diversion'}});
}

sub mapSTUFF {
    my ($self, $data, $event, $element) = @_;
    my ($parent) = $data->{'diversion'}[-1];

    my ($gen_object) = new Text::GDO::Spec::Thing;
    $gen_object->{'parent'} = $parent;
    $gen_object->{'source'} = $event;
    push (@{$data->{'diversion'}}, $gen_object);
    $self->_map_instance ($data, $event);
    $parent->{'stuff'} = pop (@{$data->{'diversion'}});
}

sub mapATTR {
    my ($self, $data, $event, $element) = @_;
    my ($parent) = $data->{'diversion'}[-1];

    my ($gen_object) = new Text::GDO::Spec::Attr;
    $gen_object->{'parent'} = $parent;
    $gen_object->{'source'} = $event;
    push (@{$data->{'diversion'}}, $gen_object);
    $self->_map_instance ($data, $event);
    push (@{$parent->{'content'}}, pop (@{$data->{'diversion'}}));
}

1;
